@@ -9,9 +9,12 @@
 ######################################################
 
 import argparse
+import glob
+import os
 import random
 import re
 import requests
+import requests_cache
 import sys
 
 try:
@@ -19,6 +22,11 @@ try:
 except ImportError:
     from urllib import quote as url_quote
 
+try:
+    from urllib import getproxies as get_proxies
+except ImportError:
+    from urllib.request import getproxies as get_proxies
+
 from pygments import highlight
 from pygments.lexers import guess_lexer, get_lexer_by_name
 from pygments.formatters import TerminalFormatter
@@ -44,9 +52,11 @@ USER_AGENTS = ('Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:11.0) Gecko/2010
                'Mozilla/5.0 (Windows; Windows NT 6.1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.46 Safari/536.5',)
 ANSWER_HEADER = u('--- Answer {0} ---\n{1}')
 NO_ANSWER_MSG = '< no answer given >'
+CACHE_DIR = os.path.join(os.path.expanduser('~'), '.howdoi')
+CACHE_FILE = os.path.join(CACHE_DIR, 'cache')
 
 def get_result(url):
-    return requests.get(url, headers={'User-Agent': random.choice(USER_AGENTS)}).text
+    return requests.get(url, headers={'User-Agent': random.choice(USER_AGENTS)}, proxies=get_proxies()).text
 
 
 def is_question(link):
@@ -149,6 +159,17 @@ def get_instructions(args):
     return '\n'.join(answers)
 
 
+def enable_cache():
+    if not os.path.exists(CACHE_DIR):
+        os.makedirs(CACHE_DIR)
+    requests_cache.install_cache(CACHE_FILE)
+
+
+def clear_cache():
+    for cache in glob.glob('{0}*'.format(CACHE_FILE)):
+        os.remove(cache)
+
+
 def howdoi(args):
     args['query'] = ' '.join(args['query']).replace('?', '')
     try:
@@ -159,7 +180,7 @@ def howdoi(args):
 
 def get_parser():
     parser = argparse.ArgumentParser(description='instant coding answers via the command line')
-    parser.add_argument('query', metavar='QUERY', type=str, nargs='+',
+    parser.add_argument('query', metavar='QUERY', type=str, nargs='*',
                         help='the question to answer')
     parser.add_argument('-p','--pos', help='select answer in specified position (default: 1)', default=1, type=int)
     parser.add_argument('-a','--all', help='display the full text of the answer',
@@ -169,13 +190,32 @@ def get_parser():
     parser.add_argument('-c', '--color', help='enable colorized output',
                         action='store_true')
     parser.add_argument('-n','--num-answers', help='number of answers to return', default=1, type=int)
+    parser.add_argument('-C','--clear-cache', help='clear the cache',
+                        action='store_true')
+
     return parser
 
 
 def command_line_runner():
     parser = get_parser()
     args = vars(parser.parse_args())
-    print(howdoi(args))
+
+    if args['clear_cache']:
+        clear_cache()
+        print('Cache cleared successfully')
+        return
+
+    if not args['query']:
+        parser.print_help()
+        return
+
+    # enable the cache if user doesn't want it to be disabled
+    if not os.getenv('HOWDOI_DISABLE_CACHE'):
+        enable_cache()
+
+
+    print(howdoi(args).encode('utf-8', 'ignore'))
+
 
 if __name__ == '__main__':
     command_line_runner()
